local playerClass = select(2, UnitClass("player"))
local spellId, spellName, spellTexture, tooltip, queuedName, queuedCName
local assigned = UnitName("player")

if playerClass == "ROGUE" then
	spellId = 57934 -- Tricks of the Trade
elseif playerClass == "HUNTER" then
	spellId = 34477 -- Misdirection
elseif playerClass == "SHAMAN" then
	spellId = 974 -- Earth Shield
elseif playerClass == "PRIEST" then
	spellId = 6346 -- Fear Ward
elseif playerClass == "WARRIOR" then
	spellId = 50720 -- Vigilance
elseif playerClass == "DEATHKNIGHT" then
	spellId = 49016 -- Unholy Frenzy
else
	return
end

--------------------------------------------------------------------------------
-- Localization

local AL = LibStub("AceLocale-3.0")

local L = AL:NewLocale("EasyButton", "enUS", true)
if L then
	L["No player set"] = true
	L["%s will be placed on %s"] = true
	L["In combat, setting has been queued."] = true
end

L = AL:GetLocale("EasyButton")

--------------------------------------------------------------------------------
-- Addon

spellName, _, spellTexture = GetSpellInfo(spellId)

local icon = CreateFrame("Button", "EasyButton", UIParent, "SecureActionButtonTemplate")
icon.macroFormat = "/cast [@%s] "..spellName

icon:RegisterEvent("PLAYER_TALENT_UPDATE")

local function setMacro(name, cName)
	assigned = name
	DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99EasyButton|r: "..L["%s will be placed on %s"]:format(GetSpellLink(spellName), cName))
	icon:SetAttribute("macrotext", icon.macroFormat and icon.macroFormat:format(name) or L["No player set"])
end

local function eventHandler(self, event, ...)
	if event == "PLAYER_TALENT_UPDATE" then
		if (playerClass == "WARRIOR" and select(5, GetTalentInfo(3, 16)) == 0)
		or (playerClass == "DEATHKNIGHT" and select(5, GetTalentInfo(3, 8)) == 0)
		or (playerClass == "SHAMAN" and GetPrimaryTalentTree() ~= 3) then
			icon:Hide()
		else
			icon:Show()
		end
	elseif event == "PLAYER_REGEN_ENABLED" then
		if queuedName and queuedCName then
			setMacro(queuedName, queuedCName)
		end
		queuedName, queuedCName = nil, nil
		icon:UnregisterEvent("PLAYER_REGEN_ENABLED")
	end
end

local background = icon:CreateTexture("EasyButtonBackground", "BACKGROUND")
background:SetTexture(spellTexture)
background:SetAllPoints()

icon:SetPushedTexture('Interface\\Buttons\\UI-Quickslot-Depress')
icon:SetHighlightTexture('Interface\\Buttons\\ButtonHilight-Square')
icon:SetWidth(36)
icon:SetHeight(36)
icon:SetScale(0.78)

icon:SetClampedToScreen(true)
icon:RegisterForClicks("AnyUp")
icon:RegisterForDrag("LeftButton")
icon:SetMovable(true)
icon:SetPoint("CENTER", UIParent, "CENTER")
icon:Show()
icon:SetAttribute("type1", "macro")
icon:SetAttribute("macrotext", "")
icon:SetAttribute("type2", "macro")
icon:SetAttribute("macrotext2", "/script EasyButton:CreateTip()")

local LibQTip = LibStub('LibQTip-1.0')

local function createUnitRow(unit)
	local y, x = tooltip:AddLine()
	local name = UnitName(unit)
	if assigned == name then
		tooltip:SetCell(y, 1, "|TInterface\\Buttons\\UI-CheckBox-Check:20:20|t")
	else
		tooltip:SetCell(y, 1, "|TInterface\\Buttons\\UI-CheckBox-UP:20:20|t")
	end
	
	local role = UnitGroupRolesAssigned(unit)
	if GetPartyAssignment("MAINTANK", unit) then
		tooltip:SetCell(y, 2, "|TInterface\\GroupFrame\\UI-Group-MainTankIcon:0|t")
	elseif GetPartyAssignment("MAINASSIST", unit) then
		tooltip:SetCell(y, 2, "|TInterface\\GroupFrame\\UI-Group-MainAssistIcon:0|t")
	elseif role == "TANK" then
		tooltip:SetCell(y, 2, "|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES.blp:20:20:0:0:64:64:0:19:22:41|t")
	elseif role == "HEALER" then
		tooltip:SetCell(y, 2, "|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES.blp:20:20:0:0:64:64:20:39:1:20|t")
	elseif role == "DAMAGER" then
		tooltip:SetCell(y, 2, "|TInterface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES.blp:20:20:0:0:64:64:20:39:22:41|t")
	else
		tooltip:SetCell(y, 2, "")
	end

	local c = RAID_CLASS_COLORS[select(2, UnitClass(unit))]
	if unit:match("%d*pet%d*$") then c = {r=1,b=1,g=1} end
	local cName = format("|cff%02x%02x%02x%s|r",c.r*255,c.g*255,c.b*255,name)
	tooltip:SetCell(y, 3, cName)
	tooltip:SetLineScript(y, "OnMouseUp", function()
		if InCombatLockdown() then
			icon:RegisterEvent("PLAYER_REGEN_ENABLED")
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99EasyButton|r: "..L["In combat, setting has been queued."])
			queuedName = name
			queuedCName = cName
		else
			setMacro(name, cName)
		end
		tooltip:Hide()
	end)
end

function icon:CreateTip()
	if tooltip then
		LibQTip:Release(tooltip)
	end
	
	tooltip = LibQTip:Acquire("EasyButtonTooltip", 3, "CENTER", "CENTER", "LEFT")
	tooltip:Clear()
	icon.tooltip = tooltip
	if UnitInRaid("player") then
		for i = 1, 40 do
			if UnitPlayerControlled("raidpet"..i) then createUnitRow("raidpet"..i) end
			if UnitIsPlayer("raid"..i) then createUnitRow("raid"..i) end
		end
	else
		if UnitPlayerControlled("pet") then createUnitRow("pet") end
		createUnitRow("player")
		for i = 1, 4 do
			if UnitPlayerControlled("partypet"..i) then createUnitRow("partypet"..i) end
			if UnitIsPlayer("party"..i) then createUnitRow("party"..i) end
		end
	end
	tooltip:SetAutoHideDelay(0.01, icon)
	tooltip:SmartAnchorTo(icon)
	tooltip:Show()
end

icon:SetScript("OnEvent", eventHandler)

icon:SetScript("OnDragStart", function()
	if IsShiftKeyDown() then
		icon:StartMoving()
	end
end)

icon:SetScript("OnDragStop", function()
	icon:StopMovingOrSizing()
end)

